
#pragma once

#include "AI.h"
#include "StoneChainManager.h"
#include "JosekiDictionary.h"

class CapriccioAI : public AI
{
public:
	CapriccioAI();
	~CapriccioAI();

	virtual void SetBoardSize(int boardsize) {}	// fixed at 9
	virtual void ClearBoard();
	virtual void SetMove(Move move);
	virtual void SetKomi(float komi) {}; // fixed at 6.5
	virtual void NextMove(int color, char* moveStr);

	// info
	virtual char* GetName() { return "Capriccio"; }
	virtual char* GetVersion() { return "1.0"; }

#if defined (ENABLE_DEBUG_OUTPUT)
	// debug
	virtual void ShowBoard() { manager_.ShowBoard(); }
	virtual void ShowBitBoard() { manager_.ShowBitBoard(); }
	virtual void ShowAllChains();
#endif

private:	// structs

	typedef struct WeightedMove
	{
		Move move_;
		int  value_;
	} WeightedMove;

	static const int MAX_MOVELIST_COUNT = 1024;
	typedef struct MoveList
	{
		int				count_;
		WeightedMove	moves_[MAX_MOVELIST_COUNT];
	} MoveList;

private:	// methods
	bool WasPrevMoveAtari();
	bool GetImmediateMove(Move& move);

	bool GetYoseHaneResponseMove(Move& move);
	inline bool CheckFirstLineYoseHane(Move& move, Point pt1, Point pt2, Point pt3, Point pt4);
	inline bool CheckSecondLineOsae(Move& move, Move osaeMove, Move oppMove1, Move oppMove2, Point chainPos);

	// attack move search functions
	bool FindAttackMove_0(Move& move, StoneChain* chain);
	bool FindAttackMove_0_(Move& move, BitBoard9 libs, int numLibs);
	bool FindAttackMove_1(MoveList& moves, StoneChain* chain);
	bool FindAttackMove_1_(MoveList& moves, StoneChain *chain, BitBoard9 libs, int numLibs);
	bool FindAttackMove(MoveList& moves, StoneChain* chain, int depth);

	// defense move search functions
	bool FindDefenseMove_1(Move& move, StoneChain* chain);
	bool FindDefenseMove(MoveList& moves, StoneChain* chain, int depth);

	// search for safe making/breaking moves
	void FindSafeMakingMoves(MoveList& moves, StoneColor color);
	void FindSafeBreakingMoves(MoveList& moves, StoneColor color);

	// search for yose moves
	void FindYoseMoves(MoveList& moves, StoneColor color);

	// appends all possible moves that the ai may make to the move list.
	void AppendPossibleMoves(MoveList& moves, StoneColor color);

	// calculates the relativity value from pt to the chain
	int  GetRelativity(Point pt, StoneChain* chain);

	void UpdateChainState();

	bool CheckIfWeak(StoneChain* chain);
	bool CheckIfUrgent(StoneChain* chain);
	bool CheckIfSafe(StoneChain* chain);
	bool CheckSafeConnection(StoneChain* chain);
	bool IsSafeLibertyPair(Point pt1, Point pt2, StoneColor color);
	bool IsSafeWithOneLessLib(StoneChain *chain);
	bool IsLooseLadder(StoneChain *chain);

	bool IsStrictlySafe(Move move);

private:	// fields
	Move prevMove_;
	StoneChainManager manager_;
	JosekiDictionary* joseki_;

	// variables used to search moves
	MoveList possibleMoveList_;
	MoveList searchList_;

	// accumulates the move values
	static const int BOARD_WIDTH  = 11;
	static const int BOARD_HEIGHT = 11;
	int moveValues_[BOARD_WIDTH][BOARD_HEIGHT];
	
	// for relativity
	static const int MAX_RELATIVE_DISTANCE = 4;
	BitBoard9 distanceMask_[BOARD_WIDTH][BOARD_HEIGHT][MAX_RELATIVE_DISTANCE];
	int relativity_[BOARD_WIDTH][BOARD_HEIGHT];

	BitBoard9 upperLeft33Mask_;
	BitBoard9 upperRight33Mask_;
	BitBoard9 lowerLeft33Mask_;
	BitBoard9 lowerRight33Mask_;

	enum ChainState
	{
		NONE   = 0x0,
		SAFE   = 0x1,
		WEAK   = 0x2,
		URGENT = 0x4,
	};

	ChainState chainState_[MAX_UNDOS];		// chain state, index refers to the chain index
	BitBoard9   safeLibs_[MAX_UNDOS];		// chains's safe libs, index refers to the chain index
};