
#include <new>
#include <stdio.h>
#include <iostream>

#include "GtpClient.h"
#include "BitBoard9.h"

static char buffer[1024];
static char commandStr[1024];


static char* knownCommands[] = 
{
	// required
	"protocol_version",
	"name",
	"version",
	"known_command",
	"list_commands",
	"quit",
	"boardsize",
	"clear_board",
	"komi",
	"play",
	"genmove"

	// optional
};
static int   numKnownCommands = sizeof(knownCommands) / sizeof (char*);

#define PROTOCOL_VERSION 2

GtpClient::GtpClient(AI *ai)
 : ai_(ai)
{
	command_ = (GtpCommand*)malloc(sizeof(GtpCommand));
}

GtpClient::~GtpClient()
{
	free((void*)command_);
}

bool GtpClient::ParseAndExecute()
{
	std::cin.getline(commandStr, 1024);

	command_ = new(command_) GtpCommand(commandStr);

	if (!strcmp(command_->commandName_, "genmove"))
	{
		int color = command_->arguments_[0] == 'b' ? 0 : 1;
		ai_->NextMove(color, commandStr);

		std::cout << "=" << command_->IdString_ << " " << commandStr << std::endl << std::endl;
	}
	else if (!strcmp(command_->commandName_, "protocol_version"))
	{
		std::cout << "=" << command_->IdString_ << " " << PROTOCOL_VERSION << std::endl << std::endl;
	}
	else if (!strcmp(command_->commandName_, "name"))
	{
		std::cout << "=" << command_->IdString_ << " " << ai_->GetName() << std::endl << std::endl;
	}
	else if (!strcmp(command_->commandName_, "version"))
	{
		std::cout << "=" << command_->IdString_ << " " << ai_->GetVersion() << std::endl << std::endl;
	}
	else if (!strcmp(command_->commandName_, "known_command"))
	{
		bool known = false;
		
		for (int i = 0; i < numKnownCommands; i++)
		{
			if (!strcmp(knownCommands[i], command_->arguments_))
			{
				known = true;
				break;
			}
		}

		if (known)
			std::cout << "=" << command_->IdString_ << " true" << std::endl << std::endl;
		else
			std::cout << "=" << command_->IdString_ << " false" << std::endl << std::endl;
	}
	else if (!strcmp(command_->commandName_, "list_commands"))
	{
		std::cout << "=" << command_->IdString_ << " ";
		
		for (int i = 0; i < numKnownCommands; i++)
		{
			std::cout << knownCommands[i] << std::endl;
		}
		std::cout << std::endl;
	}
	else if (!strcmp(command_->commandName_, "quit"))
	{
		std::cout << "=" << command_->IdString_ << " " << std::endl << std::endl;
		return false;	// end
	}
	else if (!strcmp(command_->commandName_, "boardsize"))
	{
		ai_->SetBoardSize(atoi(command_->arguments_));
		std::cout << "=" << command_->IdString_ << " " << std::endl << std::endl;
	}
	else if (!strcmp(command_->commandName_, "clear_board"))
	{
		ai_->ClearBoard();
		std::cout << "=" << command_->IdString_ << " " << std::endl << std::endl;
	}
	else if (!strcmp(command_->commandName_, "komi"))
	{
		ai_->SetKomi((float)atof(command_->arguments_));
		std::cout << "=" << command_->IdString_ << " " << std::endl << std::endl;
	}
	else if (!strcmp(command_->commandName_, "play"))
	{
		Move move;
		move.color   = command_->arguments_[0] == 'B' ? 0 : 1;

		if (command_->arguments_[2] == 'P')
		{
			move.point.x = 0;
			move.point.y = 0;
		}
		else
		{
			move.point.x = command_->arguments_[2] - 'A' + (command_->arguments_[2] == 'J' ? 0 : 1);
			move.point.y = command_->arguments_[3] - '1' + 1;
		}

		ai_->SetMove(move);
		std::cout << "=" << command_->IdString_ << " " << std::endl << std::endl;
	}

#if defined (ENABLE_DEBUG_OUTPUT)

	// debug command
	else if (!strcmp(command_->commandName_, "showboard"))
	{
		std::cout << "=" << command_->IdString_ << " " << std::endl;
		ai_->ShowBoard();
		std::cout << std::endl;
	}
	else if (!strcmp(command_->commandName_, "showbitboard"))
	{
		std::cout << "=" << command_->IdString_ << " " << std::endl;
		ai_->ShowBitBoard();
		std::cout << std::endl;
	}
	else if (!strcmp(command_->commandName_, "showchains"))
	{
		std::cout << "=" << command_->IdString_ << " " << std::endl;
		ai_->ShowAllChains();
		std::cout << std::endl;
	}
#endif 

	// unknown command
	else
	{
		std::cout << "?" << command_->IdString_ << " unknown command" << std::endl << std::endl;
	}

	return true;
}



GtpClient::GtpCommand::GtpCommand(char *command)
{
	char* token = strtok(command, " ");

	ID_ = atoi(token);

	if (hasID_ = (ID_ && strcmp(token, "0")))
	{
		strcpy(IdString_, token);
		commandName_ = strtok(NULL, " ");
	}
	else
	{
		commandName_ = token;
		IdString_[0] = 0;
	}

	arguments_ = strtok(NULL, " ");
}