
#include "Board.h"

#include <stdio.h>

//=============================================================================
//
//	JosekiDictionary class
//  
//	Reads a sgf file and from that file, the joseki tree is build to match
//	all possible transposition of the joseki
//
//=============================================================================

class JosekiDictionary
{
public:
	// constructor
	JosekiDictionary(const char* josekiFileName);
	// destructor
	~JosekiDictionary();

	void Initialize();
	// Let tree proceed to the next node
	// currentBoard is the board after a move is made since the last update to thisclass
	void Proceed(Board currentBoard);
	// returns a joseki move on random
	Point GetNextMove();

private:
	static const int MAX_CHILDREN = 64;

	// state flags
	static const unsigned char ROTATED_0            = 0;		// original position
	static const unsigned char ROTATED_90           = 1;		// rotated  90 degrees clockwise from ROTATED_0
	static const unsigned char ROTATED_180          = 2;		// rotated 180 degrees clockwise from ROTATED_0
	static const unsigned char ROTATED_270          = 3;		// rotated 270 degrees clockwise from ROTATED_0
	static const unsigned char MIRROR_ROTATED_0     = 4;		// horizontally mirrored from ROTATED_0
	static const unsigned char MIRROR_ROTATED_90    = 5;		// rotated  90 degrees clockwise from MIRROR_ROTATED_0
	static const unsigned char MIRROR_ROTATED_180   = 6;		// rotated 180 degrees clockwise from MIRROR_ROTATED_0
	static const unsigned char MIRROR_ROTATED_270   = 7;		// rotated 270 degrees clockwise from MIRROR_ROTATED_0
	static const unsigned char NUM_ROATATION_STATES = 8;

	// JosekiNode class, representing each node of the joseki tree
	class JosekiNode
	{
	public:
		Board		boardBeforeMove_;
		Point		move_;
		JosekiNode* parent_;
		JosekiNode* children_[MAX_CHILDREN];
		int			numChildren_;

	public:
		JosekiNode();
		~JosekiNode();

		JosekiNode* Clone();
	};

	JosekiNode* currentNode_;
	JosekiNode* rootNode_;

	unsigned char currentRotation_;

private:
	// private methods
	void BuildTree(JosekiNode* root, FILE* josekiFile, StoneColor startColor);
	Point RotatePoint(Point point, unsigned char rotation);
	Board RotateBoard(Board board, unsigned char rotation);
};