
#pragma once

//=============================================================================
//
//	StoneChain class
//  
//	Represents the stone chain.
//	The information such as the stones of the chain and their neighbors are kept as BitBoard9
//	Note that this class does not contain the liberty infomation.
//	The liberty info is obtained via the StoneChainManager class by giving it the neighbors of the chain.
//
//=============================================================================

#include "BitBoard9.h"
#include "NeighborStoneChainEnumerator.h"

class StoneChain
{
public:
	// constructor
	StoneChain(int index) { index_ = index; }

	// amalgate a stone chain to this chain
	void Amalgamate(StoneChain *chain);

	// init with a single stone. 
	void Initialize(Move move);

	int GetIndex() { return index_; }
	StoneColor GetColor() { return color_; }

	BitBoard9& GetStones() { return stones_; }
	int GetNumStones() { return stones_.CountStones(); }

	BitBoard9& GetNeighbors() { return neighbors_; }
	int GetNumNeighbors() { return neighbors_.CountStones(); }

protected:
	int index_;
	StoneColor color_;

	BitBoard9	stones_;
	BitBoard9	neighbors_;
};
